from handlers import cmmdhandler, lumidbhandler


def query(options):
    """Choose appropriate handler and call it with options.

    :param options: dict query options
    :returns: result
    :rtype: dict

    """
    hanlerfn = get_handler_fn(options)
    return hanlerfn(options)


def get_handler_fn(options):
    """Select handler function by query type

    :param options: dict query options
    :returns: handler executor function
    :rtype: function

    """
    if 'query_type' not in options:
        raise KeyError(
            'Query type is missing. Could not determine handlername.')

    t = str(options['query_type']).lower()
    if t == 'timelumi':
        return _call_cmmdhandler_lumi
    elif t == 'iovtags':
        return _call_lumidbhandler_iovtags
    elif t == 'dummy':
        raise NotImplementedError('No dummy handler yet.')

    raise KeyError('Query type did not match any handler name.')


def _call_cmmdhandler_lumi(options):
    """cmmdhandler executor - call to query cmmdhandler

    :param options: dict query options
    :returns: luminosity chart data
    :rtype: dict

    """

    return cmmdhandler.brilcalcLumiHandler(options)


def _call_lumidbhandler_iovtags(options):
    """lumidbhandler executor - call to query lumidb

    :param options: dict query options
    :returns: data
    :rtype: dict

    """

    return lumidbhandler.get_iovtags()



import subprocess
import re
import numpy as np
import os
from brilview import bvconfig


def brilcalcLumiHandler(commandargs={}):
    '''
    brilcalcargs: arguments for brilcalc lumi

    the handler decides if --byls is needed according to input time parameters
    input:
         commandargs: arguments for command
         {
           begin: str '^\d\d/\d\d/\d\d \d\d:\d\d:\d\d$|^\d{6}$|^\d{4}$'
           end: str '^\d\d/\d\d/\d\d \d\d:\d\d:\d\d$|^\d{6}$|^\d{4}$'
           timeunit: run,fill,strdate
           unit: str
           beamstatus: str (must be in 'stable beams','squeeze','flat top','adjust')
           normtag: str textbox or dropdown select
           datatag: str
           hltpath:  str //'^HLT_[\w\*\?\[\]\!]+$' reject wildcard patterns ??
           type:  str or None
           selectjson: jsonstr
           byls: boolean
           without_correction: boolean
         }

    output:
    {'status':'OK'/'ERROR',
    'data': {'fillnum': [],
                  'runnum': [],
                  'tssec': [],
                  'delivered': [],
                  'recorded':[],
                  'lsnum':[],
                  'hltpathid':[],
                  'hltpathid2name':{id:name},
                  'hltpathname2id':{name:id},
                }
    or {'message':str}
    }

    '''
    begin = ''
    end = ''
    selectjson = ''

    cmd = 'brilcalc'
    if (
            hasattr(bvconfig, 'brilcommandhandler') and
            'command' in bvconfig.brilcommandhandler and
            bvconfig.brilcommandhandler['command']
    ):
        cmd = bvconfig.brilcommandhandler['command']

    cmdargs = [
        cmd, 'lumi', '--output-style', 'csv', '--without-checkjson', '--tssec']
    if 'selectjson' not in commandargs:
        if 'begin' not in commandargs or 'end' not in commandargs:
            return 'ERROR: parameters begin, end or selectjson are missing'
        else:
            begin = commandargs['begin']
            end = commandargs['end']
            cmdargs += ['--begin', str(begin), '--end', str(end)]
    else:
        selectjson = commandargs['selectjson']
        cmdargs += ['-i', selectjson]

    byls = commandargs.get('byls', False)
    if byls:
        cmdargs.append('--byls')

    without_correction = commandargs.get('without_correction', False)
    if without_correction:
        cmdargs.append('--without-correction')

    hltpath = None

    unit = '/ub'
    if 'unit' in commandargs and commandargs['unit']:
        unit = commandargs['unit']
    cmdargs += ['-u', unit]

    if 'type' in commandargs and commandargs['type']:
        cmdargs += ['--type', commandargs['type']]

    if 'normtag' in commandargs and commandargs['normtag']:
        normtag = str(commandargs['normtag']).replace('.', '').replace('/', '')
        if normtag == '' or os.path.isfile(normtag):
            return {'status': 'ERROR', 'message': 'Bad normtag: ' + normtag}
        cmdargs += ['--normtag', normtag]

    if 'beamstatus' in commandargs and commandargs['beamstatus']:
        cmdargs += ['-b', commandargs['beamstatus'].upper()]

    if 'hltpath' in commandargs and commandargs['hltpath']:
        cmdargs += ['--hltpath', commandargs['hltpath']]
        hltpath = commandargs['hltpath']

    print cmdargs
    try:
        r = subprocess.check_output(cmdargs, stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as e:
        if e.returncode != 0:
            return {'status': 'ERROR', 'message': e.output}
    result_strarray = [
        l for l in r.split('\n') if len(l) > 0 and not l.startswith('#')]

    if not result_strarray:  # no data found
        return {'status': 'ERROR', 'message': 'No data found'}
    iserror = False
    fillnums = []
    runnums = []
    lsnums = []
    tssecs = []
    delivereds = []
    recordeds = []
    hltpathids = []
    hltpathid2name = {}
    hltpathname2id = {}
    allpaths = []
    for line in result_strarray:
        items = line.split(',')
        if items[0].find(':') == -1:  # output is an error message because the first data field always n:m
            iserror = True
            break

        [runnum, fillnum] = [int(x) for x in items[0].split(':')]

        fillnums.append(fillnum)
        runnums.append(runnum)

        # special treat hltpathname field
        if byls:
            if hltpath:
                pathname = items[3]
                if pathname not in allpaths:
                    allpaths.append(pathname)
                hltpathid = allpaths.index(pathname)
                hltpathids.append(hltpathid)
                hltpathid2name[hltpathid] = pathname
                hltpathname2id[pathname] = hltpathid
                del items[3]  # delete hltpath field
            else:
                del items[3:5]  # delete beamstatus, E fields
        else:
            del items[2:4]  # delete nls,ncms or ncms,hltpath for hlt

        if byls:
            tssecs.append(int(items[2]))
            lsnum = int(items[1].split(':')[0])
            lsnums.append(lsnum)
            delivereds.append(float(items[3]))
            recordeds.append(float(items[4]))
        else:
            tssecs.append(int(items[1]))
            delivereds.append(float(items[2]))
            recordeds.append(float(items[3]))

    if iserror:
        return {'status': 'ERROR', 'message': '\n'.join(result_strarray)}
    resultdata = {
        'fillnum': fillnums,
        'runnum': runnums,
        'lsnum': lsnums,
        'tssec': tssecs,
        'delivered': delivereds,
        'recorded': recordeds,
        'hltpathid': hltpathids,
        'hltpathid2name': hltpathid2name,
        'hltpathname2id': hltpathname2id
    }
    return {'status': 'OK', 'data': resultdata}


def brilcalcBXLumiHandler(brilcalcargs, unit='/ub', cmmd=[]):
    '''
    output:
       {'status':'OK'/'ERROR',
        'data': [ [fillnum,runnum,lsnum,tssec,bxid_array,bxdelivered_array,bxrecorded_array] ]
        or 'errormessagestring'
    }
    '''
    if not brilcalcargs:
        return {'status': 'ERROR', 'data': 'Empty input command'}
    args = []
    if cmmd:
        args = cmmd
    else:
        args = ['brilcalc']
    xingMin = 0.001
    args += ['lumi'] + brilcalcargs
    args += [
        '-u', unit, '--xing', '--output-style', 'csv', '--without-checkjson',
        '--tssec', '--xingMin', str(xingMin)
    ]
    try:
        r = subprocess.check_output(args, stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as e:
        if e.returncode != 0:
            return {'status': 'ERROR', 'data': e.output}
    result_strarray = [
        l for l in r.split('\n') if len(l) > 0 and not l.startswith('#')
    ]
    if not result_strarray:  # no data found
        return {'status': 'ERROR', 'data': 'No data found'}
    resultdata = []
    iserror = False
    for line in result_strarray:
        items = line.split(',')
        if items[0].find(':') == -1:  # output is an error message
            iserror = True
            break
        [runnum, fillnum] = [int(x) for x in items[0].split(':')]
        [lsnum, cmsalive] = [int(x) for x in items[1].split(':')]
        tssec = int(items[2])
        bxlumi_str = items[9]
        bxlumi_str = re.sub(r'\[|\]', '', bxlumi_str)
        bxlumi = np.fromstring(bxlumi_str, dtype='f4', sep=' ')
        bxid_array = bxlumi[0::3].astype(int)
        bxdelivered_array = bxlumi[1::3]
        bxrecorded_array = bxlumi[2::3]
        resultdata.append([
            fillnum, runnum, lsnum, tssec, bxid_array.tolist(),
            bxdelivered_array.tolist(), bxrecorded_array.tolist()
        ])
    if iserror:
        return {'status': 'ERROR', 'data': '\n'.join(result_strarray)}
    return {'status': 'OK', 'data': resultdata}


def brilcalcBeamHandler(brilcalcargs, cmmd=[]):
    pass


def brilcalcBXBeamHandler(brilcalcargs, cmmd=[]):
    pass


if __name__ == '__main__':
    # from project root run python -m brilview.handlers.cmmdhandler
    # print brilcalcLumiHandler(
    #    ['-r', '284077'], unit='/mb',cmmd=['/home/zhen/work/brilws/brilcalc-run.py'])
    # print brilcalcBXLumiHandler(
    #    ['-r', '284077'], unit='/mb',cmmd=['/home/zhen/work/brilws/brilcalc-run.py'])
    # print brilcalcLumiHandler(
    #    ['-r', '284077'],
    #    cmmd=['/home/data/brilws/brilcalc-run.py'])
    bvconfig.update({"brilcommandhandler": {}})
    bvconfig.brilcommandhandler['command'] = '/home/zhen/work/brilws/brilcalc-run.py'
    print brilcalcLumiHandler({'begin': 284077, 'end': 284077, 'byls': True})



docstr = """Usage: brilview [options]

Options:
     -h, --help                          Show this screen
     --debug                             Debug mode
     --daemon                            Daemon mode
     --without-browser                   Run application server only
     --config CONFIG                     Server config file
     --host HOST                         host name
     --port PORT                         port
     --pidfile PIDFILE                   pidfile
     --user USER                         user
     --group GROUP                       group

"""
import sys
import os
import docopt
import webbrowser
import yaml
import cherrypy as cp
import brilview
from cherrypy.process import plugins
from brilview import brilview_flask_app, bvlogging, bvconfig

BRILVIEW_PYTHONPATH = (
    os.path.dirname(sys.executable) + '/../lib/python2.7/site-packages/')
# ignore other PYTHONPATH
sys.path.insert(0, BRILVIEW_PYTHONPATH)

THISPATH = os.path.abspath(os.path.dirname(__file__))

log = bvlogging.get_logger()


def run_in_cp_tree(app, after_start=None):

    cpconfig = {
        'engine.autoreload.on': False
    }
    if hasattr(bvconfig, 'host') and bvconfig.host is not None:
        cpconfig['server.socket_host'] = bvconfig.host
    if hasattr(bvconfig, 'port') and bvconfig.port is not None:
        cpconfig['server.socket_port'] = bvconfig.port
    if hasattr(bvconfig, 'log_to_screen'):
        cpconfig['log.screen'] = bvconfig.log_to_screen
    if hasattr(bvconfig, 'log_file') and bvconfig.log_file is not None:
        cpconfig['log.error_file'] = bvconfig.log_file

    cp.config.update(cpconfig)

    if hasattr(bvconfig, 'daemon') and bvconfig.daemon:
        cpconfig['log.screen'] = False
        cp.config.update(cpconfig)
        plugins.Daemonizer(cp.engine).subscribe()
    cp.engine.signals.subscribe()
    if hasattr(bvconfig, 'user') or hasattr(bvconfig, 'group'):
        user = bvconfig.user if hasattr(bvconfig, 'user') else None
        group = bvconfig.group if hasattr(bvconfig, 'group') else None
        plugins.DropPrivileges(cp.engine, uid=user, gid=group).subscribe()
    if hasattr(bvconfig, 'pid_file') and bvconfig.pid_file is not None:
        plugins.PIDFile(cp.engine, bvconfig.pid_file).subscribe()

    cp.tree.graft(app, script_name='/')
    try:
        cp.engine.start()
    except:
        sys.exit(1)
    else:
        if after_start is not None:
            after_start()
        cp.engine.block()


def brilview_main(progname=sys.argv[0]):
    argv = sys.argv[1:]
    args = docopt.docopt(
        docstr, argv, help=True,
        version=brilview.__version__, options_first=True)
    log.debug('global arguments: {}'.format(str(args)))

    # if no config file is specified, use the default one in directory ../data
    if args['--config']:
        configfile = os.path.abspath(args['--config'])
    else:
        configfile = os.path.abspath(
            os.path.join(THISPATH, '..', 'data', 'brilview_user.yaml'))

    config = {}
    with open(configfile, 'r') as f:
        try:
            config = yaml.load(f)
        except yaml.YAMLError as exc:
            log.error(exc)

    if args['--debug']:
        config['log_level'] = 'DEBUG'
    if args['--daemon']:
        config['daemon'] = True
    if args['--without-browser']:
        config['open_browser'] = False
    if args['--host']:
        config['host'] = args['--host']
    if args['--port']:
        config['port'] = int(args['--port'])
    if args['--pidfile']:
        config['pid_file'] = args['--pidfile']
    if args['--user']:
        config['user'] = args['--user']
    if args['--group']:
        config['group'] = args['--group']
    if 'host' not in config:
        config['host'] = '127.0.0.1'
    if 'instance_path' in config and config['instance_path'] is not None:
        if not os.path.isabs(config['instance_path']):
            raise ValueError('instance_path attribute must be absolute path')
    else:
        config['instance_path'] = os.path.join(THISPATH, '..')

    if 'log_file' in config and config['log_file'] is not None:
        if not os.path.isabs(config['log_file']):
            config['log_file'] = os.path.join(
                config['instance_path'], config['log_file'])
    if 'pid_file' in config and config['pid_file'] is not None:
        if not os.path.isabs(config['pid_file']):
            config['pid_file'] = os.path.join(
                config['instance_path'], config['pid_file'])

    bvconfig.update(config)
    bvlogging.update(config)
    log.debug(str(bvconfig.get_dict()))

    app = brilview_flask_app.init_app()

    if 'open_browser' in config and config['open_browser']:
        def after_start():
            url = 'http://'+config['host'] + ':' + str(config['port'])
            log.debug('Opening default browser at {}'.format(url))
            webbrowser.open(url)
    else:
        def after_start():
            pass
        
    if 'use_cherrypy' in config and config['use_cherrypy']:
        run_in_cp_tree(app, after_start=after_start)
    else:
        brilview_flask_app.run(after_start=after_start)

    return



import logging
import logging.handlers

formatter = logging.Formatter(
    '%(asctime)s %(levelname)s [%(filename)s:%(lineno)d]: %(message)s')

stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
file_handler = logging.NullHandler()
logger = logging.getLogger('CENTRAL_BRILVIEW_LOGGER')
logger.addHandler(stream_handler)
logger.addHandler(file_handler)


def update(config):
    global stream_handler
    global file_handler

    logger.removeHandler(stream_handler)
    logger.removeHandler(file_handler)

    if 'log_level' in config:
        level = logging.getLevelName(config['log_level'])
    else:
        level = logging.getLevelName('INFO')

    if 'log_to_screen' in config and config['log_to_screen']:
        stream_handler.close()
        stream_handler = logging.StreamHandler()
        stream_handler.setLevel(level)
        stream_handler.setFormatter(formatter)
    else:
        stream_handler.close()
        stream_handler = logging.NullHandler()

    if 'log_file' in config and config['log_file'] is not None:
        if 'log_file_max_bytes' in config:
            maxbytes = config['log_file_max_bytes']
        else:
            maxbytes = 10485760
        if 'log_file_backup_count' in config:
            backupcount = config['log_file_backup_count']
        else:
            backupcount = 5

        file_handler.close()
        file_handler = logging.handlers.RotatingFileHandler(
            config['log_file'], maxBytes=maxbytes, backupCount=backupcount)
        file_handler.setLevel(level)
        file_handler.setFormatter(formatter)
    else:
        file_handler.close()
        file_handler = logging.NullHandler()

    logger.setLevel(level)
    logger.addHandler(stream_handler)
    logger.addHandler(file_handler)


def get_logger():
    return logger


def get_current_handlers():
    return [stream_handler, file_handler]



from brilview import bvconfig
from ConfigParser import SafeConfigParser
from distutils.spawn import find_executable
import sqlalchemy as sql
import os


DEFAULT_ENGINE = None


def parseservicemap(authfile):
    """Parse service config ini file

    :param authfile: file path
    :returns: parsed service map
    :rtype: {servicealias:[protocol,user,passwd,descriptor]}

    """
    result = {}
    parser = SafeConfigParser()
    parser.read(authfile)
    for s in parser.sections():
        protocol = parser.get(s, 'protocol')
        user = parser.get(s, 'user')
        passwd = parser.get(s, 'pwd')
        descriptor = parser.get(s, 'descriptor')
        result[s] = [protocol, user, passwd, descriptor]
    return result


def create_engine(servicemap, servicename):
    user = servicemap[servicename][1]
    passwd = servicemap[servicename][2].decode('base64')
    descriptor = servicemap[servicename][3]
    connurl = 'oracle+cx_oracle://{}:{}@{}'.format(user, passwd, descriptor)
    return sql.create_engine(connurl)


def get_engine(use_cached=True):
    if use_cached and DEFAULT_ENGINE is not None:
        return DEFAULT_ENGINE
    authfile = None
    servicename = 'online'
    if hasattr(bvconfig, 'lumidbhandler'):
        if (
                'authfile' in bvconfig.lumidbhandler and
                bvconfig.lumidbhandler['authfile']
        ):
            authfile = bvconfig.lumidbhandler['authfile']
        if (
                'connection' in bvconfig.lumidbhandler and
                bvconfig.lumidbhandler['connection']
        ):
            servicename = bvconfig.lumidbhandler['connection']
    if authfile is None:
        whichbrilcalc = find_executable('brilcalc')
        if whichbrilcalc is not None:
            authfile = os.path.join(
                os.path.dirname(whichbrilcalc), '..',
                'lib/python2.7/site-packages/brilws/data/readdb3.ini')
    if authfile is None:
        raise RuntimeError('Cannot create SQL engine without authfile path')
    servicemap = parseservicemap(authfile)
    print(servicemap)
    return create_engine(servicemap, servicename)


def get_iovtags():
    return _get_iovtags(get_engine())


def _get_iovtags(engine):
    resultproxy = engine.execute('select tagname from cms_lumi_prod.iovtags')
    return [t[0] for t in resultproxy.fetchall()]


if __name__ == '__main__':
    servicemap = parseservicemap('../data/db_read.ini')
    engine = create_engine(servicemap, 'online')
    print(_get_iovtags(engine))
    print(get_iovtags())



import numpy as np
import numbers
import calendar, datetime

#class ini_parser(ConfigParser.SafeConfigParser):
#    '''
#    extended SafeConfigParser class adding as_dict method
#    '''
#    def as_dict(self):
#        d = dict(self._sections)
#        for k in d:
#            d[k] = dict(self._defaults, **d[k])
#            d[k].pop('__name__', None)
#        return d

def blob_to_array(iblob,itypecode):
    '''
    unpack blob(buffer) to c array
    '''
    if not isinstance(iblob,buffer) and not isinstance(iblob,str):
        return None
    if itemtypecode not in ['c','b','B','u','h','H','i','I','l','L','f','d']:
        raise RuntimeError('unsupported typecode '+itemtypecode)
    result=array.array(itemtypecode)

    if not iblob :
        return None
    result.fromstring(iblob)
    return result

def _find_datebins(tsarray, bintype='day'):
    '''
    Digitize input array by date type (day, week, month,year).
    input: list of unixtimestamp
    output:
    [[pos_in_tsarray),],[],[]]
   
    '''
    list_of_dts = [ ( i,datetime.datetime.utcfromtimestamp(x) )  for (i,x) in enumerate(tsarray) ]
    data = {}
    for (i,dt) in list_of_dts :
        if bintype=='day':
            data.setdefault( dt.toordinal(), [] ).append( i )
        elif bintype=='week':
            data.setdefault( dt.isocalendar()[0]*100+dt.isocalendar()[1],[] ).append( i )
        elif bintype=='month':
            data.setdefault( dt.year*100+dt.month,[] ).append( i )
        elif bintype=='year':
            data.setdefault( dt.year,[] ).append( i )
    results = [ data.get(x, []) for x in range(min(data), max(data)+1) ]
    return results

def _find_numbins(numarray,nbins=None,step_size=None):
    '''
    Digitize input number list by number of bins or step size
    output:
    [[pos_in_numarray,],[],[]]
      or None is input array is not digitized
    '''
    step_count = 0
    bins = 100 
    if nbins:
        bins = nbins
    elif step_size:
        bins = np.arange( min(numarray),max(numarray)+step_size, step_size )
    if isinstance(bins, numbers.Number) :
        if bins > len(numarray):
            return None # not aggregated    
    else:
        if bins.size > len(numarray): # no need of aggregation
            return None
            
    ( hist, bin_edges ) = np.histogram( np.array(numarray), bins=bins )
    results = []
    for i,e in enumerate( bin_edges[:-1] ):
        left_edge = e
        right_edge = bin_edges[i+1]
        pos = []
        if i==(bin_edges.size-2):#if last bin, the right edge is inclusive 
            pos = np.where( (numarray<=right_edge)&(numarray>=left_edge) )[0]
        else:
            pos = np.where( (numarray<right_edge)&(numarray>=left_edge) )[0]
        results.append( list(pos) )
    return results

def aggregate_1d(inputdata, bindef, aggalgos):
    '''
    inputs:
      inputdata: {
        'time':...
        'val1': ...
     }
     bindef: {
        field: str,
        step_size: int | None,
        step_unit: str (number,week,month,day) | None,
        step_count: int |None,        
     }
     aggalgos: {
           "<fieldname>": str, # left, right, middle, min, max, avg, sum
           ...
     }

    outputs:
    {
         'aggregated': True/False
         'data':{
             field: array
        }
    }
    '''
    if not bindef.has_key('field'):
        raise KeyError('field not found in the bindef input, cannot decide on the binning variable')
    binning_field = bindef['field']    
    binning_array = inputdata[binning_field]
    step_unit = 'number'
    pos = None
    result = {}
    if bindef.has_key('step_unit') and bindef['step_unit']:
        step_unit = bindef['step_unit']
    if step_unit =='number':
        pos = _find_numbins(binning_array,nbins=bindef.get('step_count',None),step_size=bindef.get('step_size',None))
    elif step_unit in ['day','week','month','year']:
        pos = _find_datebins(binning_array,bintype=step_unit)
    if not pos:
        result['aggregated'] = False
        return result
    result['aggregated'] = True
    result['data'] = {}
    for posbucket in pos:
        if not posbucket:
            result['data'].setdefault(binning_field,[]).append(None)
            continue
        leftedge = posbucket[0]
        result['data'].setdefault(binning_field,[]).append( binning_array[leftedge] ) #left
        
    for posbucket in pos:
        for fieldname in aggalgos.keys():
            if not posbucket: #empty bin
                result['data'].setdefault(fieldname,[]).append(None)  
                continue
            aggalgo = aggalgos[fieldname]
            toaggregate = [ inputdata[fieldname][i] for i in posbucket ]
            if aggalgo=='average':                
                result['data'].setdefault(fieldname,[]).append( np.average(toaggregate) )
            elif aggalgo=='sum':
                result['data'].setdefault(fieldname,[]).append( np.sum(toaggregate) )
            elif aggalgo=='max':
                result['data'].setdefault(fieldname,[]).append( np.max(toaggregate) )
            elif aggalgo=='min':
                result['data'].setdefault(fieldname,[]).append( np.min(toaggregate) )
            elif aggalgo=='left':
                result['data'].setdefault(fieldname,[]).append( toaggregate[0] )
            elif aggalgo=='right':
                result['data'].setdefault(fieldname,[]).append( toaggregate[-1] )
            elif aggalgo=='middle':
                if len(toaggregate)==1:
                    result['data'].setdefault(fieldname,[]).append( toaggregate[0] )
                elif len(toaggregate)%2==0:
                    result['data'].setdefault(fieldname,[]).append( toaggregate[ len(toaggregate)/2-1 ] )
                else:
                    result['data'].setdefault(fieldname,[]).append( toaggregate[ len(toaggregate)/2 ] )
    return result

if __name__=='__main__':        

    tt = [1452242977,1483865377,1484038177,1484045377,1484131777,1484149777,1484749869,1468852268,1474209068,1473949868]
    result =_find_datebins(tt, bintype='month')
    print 'date result ',result

    nn =  [0.2, 0.3, 0.4, 1.1, 1.2, 3.3, 3.0, 4.1, 6.2, 16.1]
    result = _find_numbins(nn,nbins=None,step_size=3)
    print 'num result by step size ',result
    result = _find_numbins(nn,nbins=5,step_size=None)
    print 'num result by nbins ',result

    inputdata = {
        'time': tt,
        'velocity': nn        
    }
    bindef = {
        'field': 'time',
        'step_unit': 'month'        
    }
    
    aggalgos = {
        'velocity': 'max'
    }
    result = aggregate_1d(inputdata, bindef, aggalgos)
    print result

    bindef = {
        'field': 'velocity',
        'step_unit': 'number',
        'step_count': 5
    }
    aggalgos = {
        'time': 'max'
    }
    result = aggregate_1d(inputdata, bindef, aggalgos)
    print result



import time
import threading
import flask
import json
import logging
from brilview import bvconfig, queryrouter, bvlogging


# static_url_path is hardcoded because it cannot be changed after creating app,
# but there should not be need to make it different than empty string
app = flask.Flask(__name__, static_url_path='')


def init_app():
    app.instance_path = bvconfig.instance_path
    app.logger.handlers = []
    for h in bvlogging.get_current_handlers():
        app.logger.addHandler(h)

    if hasattr(bvconfig, 'flask'):
        app.config.update(bvconfig.flask)
        # workaround: static_folder is not picked from app.config
        if 'static_folder' in bvconfig.flask:
            app.static_folder = bvconfig.flask['static_folder']
    return app


@app.route('/')
def root():
    # return app.send_static_file('Default.htm')
    return app.send_static_file('index.html')


@app.route('/api/query', methods=['GET', 'POST'])
@app.route('/query', methods=['GET', 'POST'])
def query():
    data = flask.request.json
    if data is None:
        return ('Bad request. Query body must be not empty.', 400)
    result = queryrouter.query(data)
    return flask.Response(json.dumps(result), mimetype='application/json')


def run_flask():
    app.run(host=bvconfig.host, port=bvconfig.port)


def run(after_start=None):
    flask_thread = threading.Thread(target=run_flask)
    flask_thread.daemon = True
    flask_thread.start()

    try:
        # FIXME: this is fake after start implementation. It just waits and
        # executes anyway. It should execute only when and immediately after
        # flask is started succesfully
        time.sleep(1)
        # First swap `werkzeug` logger handlers with bvlogging current handlers
        wzlog = logging.getLogger('werkzeug')
        wzlog.handlers = []
        for h in bvlogging.get_current_handlers():
            wzlog.addHandler(h)
        # then continue with `after_start`
        after_start()

        while True:
            time.sleep(20)

    except KeyboardInterrupt:
        return


if __name__ == '__main__':
    run()



import sys

REQUIRED = []
NOT_ALLOWED = ['update', 'get_dict']
KEYS = set()


def update(config):

    for req in REQUIRED:
        if req not in config:
            raise KeyError(
                'Cannot update config - missing required key: {}'.format(req))

    for na in NOT_ALLOWED:
        if na in config:
            raise KeyError(
                'Cannot update config - not allowed key: {}'.format(na))

    for k, v in config.iteritems():
        KEYS.add(k)
        setattr(sys.modules[__name__], k, v)


def get_dict():
    return {k: getattr(sys.modules[__name__], k) for k in KEYS}



import flask
import threading
import requests
import webbrowser
import time
import os


default_instancepath = os.path.abspath(os.path.join(os.path.dirname(__file__),'data'))
app = flask.Flask(__name__, instance_path=default_instancepath , instance_relative_config=True )

with open(os.path.join(default_instancepath,brilview.config)) as f:
    


@app.route('/')
def hello_world():
    return 'hello world'
def run():
    app.run(debug=False, threaded=True)

if __name__=='__main__':
    flask_thread = threading.Thread(target=run)
    flask_thread.daemon = True
    flask_thread.start()
    try:
        while(requests.get('http://localhost:5000/').text != 'hello world'):
            time.sleep(1)
        webbrowser.open('http://localhost:5000/')
        while(True):
            time.sleep(10)
    except KeyboardInterrupt:
        pass        



#!/usr/bin/env python
import sys
import os
import re
import subprocess

from distutils import log
from distutils.core import Command
from setuptools import setup
from setuptools.command.sdist import sdist as _sdist

class SdistWithBuildStatic(_sdist):
    '''
        Custom sdist, checks if build_static has run and web/dist directory exists
    '''
    def make_distribution(self):
        #self.run_command('build_static')
        thispath = os.path.dirname(os.path.realpath(__file__))
        distpath = os.path.join(thispath,'brilview','web','dist')
        if not os.path.isdir(distpath) or not os.path.exists(distpath):
	    raise RuntimeError( 'Prerequisite directory %s does not exist, run python setup.py build_static to create it\n'%distpath )
        return _sdist.make_distribution(self)

class BuildStatic(Command):
    user_options = []

    def initialize_options(self):
        """Abstract method that is required to be overwritten"""
        pass
    
    def finalize_options(self):
        """Abstract method that is required to be overwritten"""
        pass

    def run(self):
        log.info("running [npm install --unsafe-perm]")
        subprocess.check_output( ['npm', 'install','--unsafe-perm'], cwd='brilview/web' )

    
with open("README.md", "rb") as f:
    long_desc = f.read().decode('utf-8')
version = re.search(
                '^__version__\s*=\s*"(.*)"',
                open('brilview/_version.py').read(),
                re.M
                ).group(1)
print version

setup(
    name = "brilview",
    author = "Jonas Daugalas,Zhen Xie",
    url = "https://github.com/jonasdaugalas/brilview",
    download_url = 'https://github.com/jonasdaugalas/brilws/tarball/'+version,
    license = "MIT",
    version = version,
    description = "bril data viewer",
    long_description = long_desc,
    packages = ['brilview', 'brilview.cli','brilview.handlers'],
    entry_points = {
        "console_scripts" : ['brilview = brilview.cli.brilview_main:brilview_main']
        },
    package_data = {'data':['brilview/data/*.yaml'],'web':['brilview/web/dist/*']},
    include_package_data = True,
    cmdclass={
        'build_static': BuildStatic, #add a build_static command
        'sdist': SdistWithBuildStatic, #make sure build_static command is run as part of sdist.
    },
)


